/* Emperor - an X windows strategy game for 1-6 players
   Copyright (C) 1998 Frank Rustemeyer.
   This file is part of the Emperor package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   If you have changes and you want them to be included in a future
   release of Emperor, please e-mail them to mambres@mud.tap.de.

   This file declares the classes for the visualisation of the game.

   98/11/30, Frank Rustemeyer <mambres@mud.tap.de>: Released.
   99/02/07, Frank Rustemeyer <mambres@mud.tap.de>: Added "Forces" Window
   99/02/09, Frank Rustemeyer <mambres@mud.tap.de>: Included cities in the Forces Window
*/

#ifndef _WNDCLASS
#define _WNDCLASS
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/xv_xrect.h>
#include <xview/notify.h>
#include "empgame.h"
#include "../xvcl/xvcl.h"
#include "../xvcl/xpm.h"

#define END NULL

class mainwndw : public window    // the main window
{   public:
	mainwndw(void);
        ~mainwndw(void);

        virtual void button_pressed(Xv_object item);
        virtual void menu_selected(char *item);
        virtual void file_open(char *path);
        virtual void file_save(char *path);
        virtual void event(Xv_Window wnd, Event *e,
                           Notify_arg arg, Notify_event_type t);

        void paint(Canvas c, Xv_Window x, Display *d, 
                   Window w, Xv_xrectlist *xr); // paint window
        void selectcity(city *ct); //called by the city window's next button
  	void invalidate(int mode);
        void setwindow(coordinate c);
        void unbird(void);
        void uncity(void);
        void unscore(void);
        void unforces(void);
	void nextplayer(void);
    private:
	void settitle(void);  
 	void select(class unit *u);
        void invalidatesquare(coordinate c);
        void invalidatearea(coordinate c1, coordinate c2);
        void invalidatemoves(void); // invalidate rest-moves display
        void findnextunit(void); // set up next-unit-button
        int make_selection(coordinate c);
        void make_move(coordinate c);

        class panel *CmdPanel;
        class canvas *canv;
        class pixmap *landxpm, *flagsxpm; 
        class pixmask *unitsxpm;
	class birdwndw *bird;
	class citywndw *ctyw;
        class scorewndw *score;
        class armywndw *forces;
	class unit * selected;
        class button * nextbutton;
        class unit * nextunit; // for Next-Unit button
        class button * stoppbutton;
        class button * infobutton;
        class button * filebutton;
        class button * zugbutton;
        class filechooser *openchooser;
        class filechooser *savechooser;
        class textview *showmoves;
        class infobox *netzzug;

	coordinate selectedpos;
        int waiting; // waiting for players in the network
    };

class birdwndw : public window
{   public:
	birdwndw(window *owner);
        ~birdwndw(void);
        virtual void event(Xv_Window wnd, Event *e,
                           Notify_arg arg, Notify_event_type t);
        virtual void destroy(void);
        void invalidate(void);
        void paint(Canvas c, Xv_Window x, Display *d, 
                   Window w, Xv_xrectlist *xr); // paint window
    private:
        window *owner;
        class canvas *canv;
    };

class citywndw : public window
{   public:
	citywndw(window *owner, city *c);
        ~citywndw(void);
	void setcity(city *c);
        virtual void button_pressed(Xv_object item);
    private:
        window *owner;
	city *ct;
        class panel *pan;
        class button *okbutton;
	class button *setbutton;
        class textview *besitzer;
        class gauge *progress;
        class listbox *unitbox;
    };

class scorewndw : public window
{   public:
        scorewndw(window *owner);
        ~scorewndw(void);
        virtual void button_pressed(Xv_object item);
    private:
        window *owner;
        void countcities(void);
        void countarea(void);
        void countarmies(void);
        class panel *pan;
        class button *citybn;
        class button *areabn;
        class button *armybn;
        class gauge *score[6];
        class imageview *flagge[6];
        int sc[7];
    };

class armywndw : public window
{   public:
        armywndw(window *owner);
        ~armywndw(void);
        virtual void button_pressed(Xv_object item);
        void update(void);
    private:
        char **extract_unit(char **pixdata, int unittype);
        char **extract_land(char **pixdata, int landtype);
        void free_array(char ** array);
        window *owner;
        class panel *pan;
        class button *okbutton;
        class imageview *flagge[6];
        class imageview *einheit[UNIT_TYPES];
        class textview *zahl[6*UNIT_TYPES];
        class imageview *stadtpic;
        class textview *stadt[6];
    };

class unloadwndw : public dialog
{   public:
        unloadwndw(window *o, transporter *t);
        ~unloadwndw(void);
        virtual void button_pressed(Xv_object item);
    private:
        class panel *pan;
        class button *okbutton;
        class listbox *ladung; 
        class window *owner;
    };

class setupwndw : public dialog 
{   public:
	setupwndw();
        ~setupwndw();
        virtual void button_pressed(Xv_object item);
        virtual void checkbox_selected(Xv_object item, int value);

    private:
        class panel *pan;
        class button *okbutton;
        class button *netbutton;
        class checkbox * playerbox[6];
        class checkbox * compbox[6];
        class textbox * playername[6];
        class imageview *flagge[6];
    };

class netwndw : public dialog
{   public:
	netwndw();
        ~netwndw();
        virtual void button_pressed(Xv_object item);

    private:
        class panel *pan;
        class button *okbutton;
        class checkbox * netbox[6];
    };

#endif
